#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <stdint.h>
//#include <termios.h>
#include <pthread.h>
//#include <sys/types.h>
//#include <sys/stat.h>
#include <fcntl.h>
#include "../head/func_extern.h"
#include "../head/mg3.h"
#include "../head/control.h"
//#include "../main/pwm_enc.h"
#include "../head/motor.h"
#include "../mirs_two.h"

#define ROT_HALF 60
#define ROT_QUATER 30
#define ROT_END 10

void key_straight(int dis){
	int c_left,c_right;
	int x_l,x_r;
	char v_l, v_r, v_lm, v_rm;
	MotorInfo m_info_l, m_info_r;

	printf("************motor_key************\n");
	v_l = 30;
	v_r = 30;
	v_lm = v_l;					/*初期速度を保存*/
	v_rm = v_r;
	motor_send_clear_packet(fd1);			/*エンコーダ値をクリア*/
	motor_send_clear_packet(fd2);
	usleep(130*1000);
	motor_send_control_packet(fd1,v_l,Kp,Ki);
	motor_send_control_packet(fd2,v_r,Kp,Ki);
	usleep(130*1000);
	c_left=0;
	c_right=0;

	while(1){
		motor_send_control_packet(fd1,v_lm,Kp,Ki);
		motor_send_control_packet(fd2,v_rm,Kp,Ki);
		usleep(130*1000);
		motor_recv_status_packet(fd1, &(m_info_l.speed), &(m_info_l.pwm), &(m_info_l.puls));
		motor_recv_status_packet(fd2, &(m_info_r.speed), &(m_info_r.pwm), &(m_info_r.puls));
		c_left=m_info_l.puls;
		c_right=m_info_r.puls;
		x_l=(int)(c_left*t_par);
		x_r=(int)(c_right*t_par);
		printf("vl = %d, vr = %d, xl = %d, xr = %d\n\n", v_lm, v_rm, x_l, x_r);
		//目標距離までまだ遠いなら		
		if( dis-x_l > KEY_QUATER && dis-x_r > KEY_QUATER){
			if(x_l - x_r > 5){								//左タイヤの移動距離が多い場合
				v_lm = v_l-5;								//左-5 : 右+5
				v_rm = v_r+5;
			}
			else if(x_r - x_l > 5){							//右タイヤの移動距離が多い場合
				v_lm = v_l+5;								//左+5 : 右-5
				v_rm = v_r-5;
			}
			else{										//左右にズレがない場合
				v_lm = v_l;								//初期速度
				v_rm = v_r;
			}
		}
		//目標距離までだいぶ近づいたら速度を1/4に
		else if( (dis-x_l < KEY_QUATER && dis-x_l > KEY_END) && (dis-x_r < KEY_QUATER && dis-x_r > KEY_END)){
			v_lm = v_l/4;
			v_rm = v_r/4;
		}
		//目標距離に到達したら終了へ
		else if( abs(dis-x_l) < KEY_END && abs(dis-x_r) < KEY_END){
			motor_send_control_packet(fd1,0,Kp,Ki);
			motor_send_control_packet(fd2,0,Kp,Ki);
			break;
		}
		else if( dis-x_l < KEY_END && dis-x_r > KEY_END ){	//左は到達している，右はまだ到達していない
			v_lm = 0;
			v_rm = 3;
		}
		else if( dis-x_l > KEY_END && dis-x_r < KEY_END ){	//左はまだ到達していない，右は到達している
			v_lm = 3;
			v_rm = 0;
		}
		//目標距離を行き過ぎたらそのタイヤをBack
		else if( x_l-dis > KEY_END && x_r-dis > KEY_END){	//左右とも行き過ぎている
			v_lm = -3;
			v_rm = -3;
		}
		else if( x_l-dis > KEY_END && (x_r-dis < KEY_END && x_r-dis > 0) ){	//左は行き過ぎ，右は行き過ぎただが許容範囲内
			v_lm = -3;
			v_rm = 0;
		}
		else if( x_l-dis > KEY_END && dis-x_r > KEY_END ){	//左は行き過ぎ，右はまだ到達していない
			v_lm = -3;
			v_rm = v_r/4;
		}
		else if( (x_l-dis < KEY_END && x_l-dis > 0) && x_r-dis > KEY_END){	//左は行き過ぎただが許容範囲内，右は行き過ぎ
			v_lm = 0;
			v_rm = -3;
		}
		else if( dis-x_l > KEY_END && x_r-dis > KEY_END){	//左はまだ到達していない，右は行き過ぎ
			v_lm = v_l/4;
			v_rm = -3;
		}
	}
	motor_send_control_packet(fd1,0,Kp,Ki);
	motor_send_control_packet(fd2,0,Kp,Ki);
	motor_send_clear_packet(fd1);
	motor_send_clear_packet(fd2);
}

void key_rotate(){
	double angle = 180*PI/180;
	int dis = (int)(angle*MIRS_RADIUS);
	int time = 1800;
	int c_left,c_right;
	int speed_l,speed_r, x_l, x_r;
	char v_l, v_r, v_lm, v_rm;
	MotorInfo m_info_l, m_info_r;

	printf("\n*****************motor_rotate*****************\n");

	speed_l = (int)(-100*dis/(t_par*time));
	speed_r = (int)(100*dis/(t_par*time));
	if(abs(speed_l) > VMAX){
		if(speed_l > VMAX){
			v_l = VMAX;
			v_r = -VMAX;
		}
		else{
			v_l = -VMAX;
			v_r = VMAX;
		}
	}
	else{
		v_l = (char)speed_l;
		v_r = (char)speed_r;
	}

	v_lm = v_l;				/*初期速度を保存*/
	v_rm = v_r;
	dis=abs(dis);				/*目標距離を正の数に変換*/

	motor_send_clear_packet(fd1);		/*エンコーダ値をクリア*/
	motor_send_clear_packet(fd2);
	usleep(130*1000);
	motor_send_control_packet(fd1,v_l,Kp,Ki);
	motor_send_control_packet(fd2,v_r,Kp,Ki);
	usleep(130*1000);
	c_left=0;
	c_right=0;

	while(1){
		motor_send_control_packet(fd1,v_lm,Kp,Ki);
		motor_send_control_packet(fd2,v_rm,Kp,Ki);
		usleep(130*1000);
		motor_recv_status_packet(fd1, &(m_info_l.speed), &(m_info_l.pwm), &(m_info_l.puls));
		motor_recv_status_packet(fd2, &(m_info_r.speed), &(m_info_r.pwm), &(m_info_r.puls));
		c_left=m_info_l.puls;
		c_right=m_info_r.puls;
		x_l=(int)(c_left*t_par);
		x_r=(int)(c_right*t_par);
		printf("vl = %d, vr = %d, xl = %d, xr = %d\n\n", v_lm, v_rm, x_l, x_r);
		//目標距離までまだ遠いなら
		if( dis+x_l > ROT_HALF && dis-x_l > ROT_HALF){
			if(x_l+x_r > 5){
				v_lm = v_l-3;
				v_rm = v_r-3;
			}
			else if(x_l+x_r < -5){
				v_lm = v_l+3;
				v_rm = v_r+3;
			}
			else{
				v_lm = v_l;
				v_rm = v_r;
			}
		}
		//目標距離まである程度近づいたら速度を±10に落とす
		else if( (dis+x_l < ROT_HALF && dis+x_l > ROT_QUATER) && (dis-x_r < ROT_HALF && dis-x_r > ROT_QUATER) ){
			v_lm = -10;
			v_rm = 10;
		}
		//目標距離までだいぶ近づいたら速度を±5に落とす
		else if( (dis+x_l < ROT_QUATER && dis+x_l > ROT_END) && (dis-x_r < ROT_QUATER && dis-x_r > ROT_END) ){
			v_lm = -5;
			v_rm = 5;
		}
		//目標距離に到達したら調整して終了へ
		else if(abs(dis+x_l) < ROT_END && abs(dis-x_r) < ROT_END){
			motor_send_control_packet(fd1,0,Kp,Ki);
			motor_send_control_packet(fd2,0,Kp,Ki);
			break;
		}
		else if(abs(dis+x_l) < ROT_END && dis-x_r > ROT_END){
			v_lm = 0;
			v_rm = 3;
		}
		else if(dis+x_l > ROT_END && abs(dis-x_r) < ROT_END){
			v_lm = -3;
			v_rm = 0;
		}
		//目標距離を行き過ぎたらそのタイヤをBack
		else if(abs(x_l) > dis+ROT_END && x_r > dis+ROT_END){	//左右とも行き過ぎている
			v_lm = 3;
			v_rm = -3;
		}
		else if(abs(x_l) > dis+ROT_END && (dis-x_r < 0 && x_r < dis+ROT_END) ){	//左は行き過ぎ，右は行き過ぎだが許容範囲内
			v_lm = 3;
			v_rm = 0;
		}
		else if(abs(x_l) > dis+ROT_END && (dis-x_r > 0 && x_r < dis-ROT_END)){		//左は行き過ぎ，右はまだ到達していない
			v_lm = 3;
			v_rm = 3;
		}
		else if((dis+x_l < 0 && abs(x_l) < dis+ROT_END) && x_r > dis+ROT_END){		//左は行き過ぎだが許容範囲内，右は行き過ぎ
			v_lm = 0;
			v_rm = -3;
		}
		else if((dis+x_l > 0 && abs(x_l) < dis-ROT_END) && x_r > dis+ROT_END){		//左はまだ到達していない，右は行き過ぎ
			v_lm = -3;
			v_rm = -3;
		}
	}
	
	motor_send_control_packet(fd1,0,Kp,Ki);
	motor_send_control_packet(fd2,0,Kp,Ki);
	motor_send_clear_packet(fd1);
	motor_send_clear_packet(fd2);
}
